
<!-- Theme Main components -->
{{scriptData}}
{{themeNamePage}}
{{scriptFilePath}}

<h1 id="entry-title">{{themeName}}</h1>
<h2>This theme is compose of {{taskNameList}}</h2>
<p>{{description}}</p>
<div id="main_container"></div>