@import url('colorStyle.css');

/* HEADER */

header{
    position: fixed;
    top: 0;
    left: 0;
    right: 0;
    z-index: 100;
    background-color: var(--tertiary-color);
    box-shadow: 0 0 5px var(--quinary-color);
    color: var(--font-color);
}

header a {
    text-decoration: none;
    color: var(--font-color);
}


 #header-grid{
    display: flex;
    justify-content: space-between;
    align-items: center;
    padding: 8px 64px 8px 64px;
    font-size: 20px;
    font-family: 'Poppins', sans-serif;
    font-weight: 700;
    margin: 0 0 0 0;
}

header .icon img{
    width: 32px;
    height: 32px;
    
    align-items: center;
}

header .icon{
    display: flex;
    align-items: center;
    margin-left: 1rem;
    margin-right: 0;
    /* padding: 0.5rem; */
}


header nav ul{
    list-style: none;
    margin: 0;
    padding: 0;
}

header nav{
    display: flex;
    justify-content: space-between;
    align-items: center;

}

header nav a{
    text-decoration: none;
    margin-inline: 8px;
    padding: 8px;
    border-radius: 10px;
    transition: background-color 0.3s ease-in-out;
}

#link-description{
    display: flex;
    font-size: 8px;
}

header nav a:hover{
    border-radius: 10px;
    background-color: var(--primary-color);
    box-shadow: rgba(0, 0, 0, 0.25) 0px 14px 28px, rgba(0, 0, 0, 0.22) 0px 10px 10px;
    transition: all 0.3s ease-in-out;
}

@media screen and (max-width: 950px){
    nav ul li a{
        display: none;
        padding: 10px;
    }
}