@import url('https://fonts.googleapis.com/css2?family=Poppins:wght@400;700&display=swap');
@import url('colorStyle.css');
@import url('bodyStyle.css');
@import url('headerStyle.css');
@import url('navigationStyle.css');
@import url('rankBarStyle.css');


/* GLOBAL PARAMETER */

/* :root{
    
} */

.main-content{
    display: flex;
    flex-direction: column;
    align-items: center;
    width: 100%;
    max-width: 1000px;
    margin: 0 auto;
    padding: 0 1rem;
}

/* GRAPHICS */

#graphics{
    display: flex;
    flex-direction: column;
    align-items: center;
    background-color: var(--white);
    color: var(--primary-color);
    border-radius: 10px;
    padding: 16px;
    box-shadow: 0 0 5px var(--box-shadow-color);
    margin: 16px;
    width: 100%;
    max-width: 70vw;
    min-width: 300px;
}

h1{
    text-decoration: none;
    margin-block: 1rem;
    display: flex;
    flex-direction: column;
    align-items: center;
}

p{
    margin: 0;
}

#chartSelector{
    display: flex;
    flex-direction: row;
    justify-content: space-around;
    align-items: center;
    padding-inline: 1em;
    padding-block: 0.5em;
    font-family: 'Poppins', sans-serif;
    background-color: aliceblue;
    border-radius: 10px;
    border: 1px solid var(--primary-color);
    font-size: large;
}

#codeSelector{
    margin-block: 1rem;
    display: grid;
    gap : 2em;
    grid-template-columns: repeat(auto-fill, minmax(100px, 1fr));
    grid-auto-rows: auto;
    width: 100%;
    

}

#codeSelector>button{
    padding-inline: 1em;
    padding-block: 0.5em;
    font-family: 'Poppins', sans-serif;
    color: var(--font-color);
    background-color: var(--quaternary-color);
}

#code-menu{
    display: flex;
    flex-direction: column;
    justify-content: space-between;
    align-items: center;
    width: 80vw;
}

#code{
    display: grid;
    grid-template-columns: 1fr 1fr;
    gap: 16px;
    font-size: 11px;
}
#code>code{
    display: none;
    background-color: var(--secondary-color);
    border-radius: 10px;
    padding: 16px;
    box-shadow: 0 0 5px var(--box-shadow-color);
    color: var(--primary-color);

    /* overflow-x : scroll; */
}

@media screen and (max-width: 1300px){
    #code{
        font-size: 10px;
    }
}
    

#compare-tool{
    display: flex;
    justify-content: space-between;
    flex-direction: column;
    align-items: center;
}

#head_elements{
    display: flex;
    flex-direction: row;
    justify-content: space-around;
    align-items: center;
    width: 100%;
}

details{
    border-radius: 10px;
    padding: 16px;
    box-shadow: 0 0 5px var(--box-shadow-color);
    margin-block: 1rem;
    margin-right: 1em;
    background-color: var(--quinary-color);
}

details>div{
    display: flex;
    flex-direction: row;
    align-items: center;
    justify-content: space-around;
    margin-block: 1rem;
}

summary{
    width: 10em;
    align-self: center;
    cursor: pointer;
    font-weight: bold;
}


/* DEBUG TOOLS */
/* you can see more clearly the box model with this */
/* div{
    background-color: var(--white);
    border-radius: 10px;
    padding: 16px;
    box-shadow: 0 0 5px var(--black);
}  */

