@import url('https://fonts.googleapis.com/css2?family=Poppins:wght@400;700&display=swap');
@import url('bodyStyle.css');
@import url('headerStyle.css');
@import url('navigationStyle.css');
@import url('rankBarStyle.css');


/* GLOBAL PARAMETER */

/* :root{
    
} */

body{
    display: flex;

    flex-direction: column;
    align-items: center;

}

.card a{
    color: var(--green6);
}


.grid{
    /* width: clamp(100px, 90%, 1200px); */
    display: grid;
    gap : 2rem;
    grid-template-columns: repeat(auto-fit, minmax(400px, 1fr));
    grid-auto-rows: auto;
    
}

.card{
    font-family: 'Poppins', sans-serif;
    font-weight: 400;
    background-color: var(--white);
    border-radius: 10px;
    box-shadow: 0 0 5px var(--box-shadow-color);
    color: var(--quinary-color);
    padding: 16px;
    display: flex;
    flex-direction: column;
    align-items: center;
    justify-content: center;
    text-align: center;
}


.description {
    font-size: 1rem;
    font-weight: 400;
    line-height: 1.5;
    margin-inline: 2rem;
}

.title{
    display: flex;
    flex-direction: row;
    align-items: center;
    gap : 2rem;
    margin-top: 1rem;
}
