@import url('bodyStyle.css');
@import url('headerStyle.css');
@import url('navigationStyle.css');

#about {
    display: flex;
    flex-direction: column;
    justify-content: center;
    align-items: center;
    margin-inline: 5em;
    padding: 0;
}
#title {
    --bg-color: 116, 150, 113;
    border-radius: 0.5em 0 0.5em 0;
    padding: 0.25em 0.5em;
    background-color: rgb(var(--bg-color));
    box-shadow: rgba(var(--bg-color), 0.4) 5px 5px, rgba(var(--bg-color), 0.3) 10px 10px, rgba(var(--bg-color), 0.2) 15px 15px, rgba(var(--bg-color), 0.1) 20px 20px, rgba(var(--bg-color), 0.05) 25px 25px;
    font-size: 2rem;
    font-weight: 700;
    margin-block: 2rem;
    text-align: center;
    color: var(--white);
}

.text-block{
    display: grid;
    
    grid-template-rows: 1fr;
    grid-template-areas: "left right";
    margin-block: 1em;
    margin-inline: 1em;
    padding: 0;
    gap : 1em;

}

.text-block div{
    display: flex;
    flex-direction: column;
    justify-content: center;
    align-items: center;
    text-align: center;
    font-size: larger;
    
}

.left{
    grid-template-columns: 40% 60%;
}

.right{
    grid-template-columns: 60% 40%;
}

.text-block img{
    width: 100%;
    height: 100%;
    object-fit: contain;
    border-radius: 10px;
}